
/*--------------------------------------------------
HTTP 1.1 Webserver as AccessPoint for ESP8266 
for ESP8266 adapted Arduino IDE

based on example by Stefan Thesen 08/2015
- free for anyone

--------------------------------------------------*/

/*--------------------------------------------------
Enhanced Version by Sven K. (oog / proog.de), 2018
- (1)
  use softAP to change IP-Address from
    192.168.4.1 (default)
  to easy to remember
    1.2.3.4
- (2)
  Switch LED_BUILTIN,
  compatible to NodeMCU Hardware, Wemos D1 Mini
- (3)
  OLED displays messages
--------------------------------------------------*/

#include <Wire.h> // I2C lib
#include "oled_ssd1306.h"

OLED oled;

#include <ESP8266WiFi.h>

const char* ssid = "ESP-IP 1.2.3.4";
const char* password = "12345678";  // set to "" for open access point w/o passwortd

IPAddress local_ip(1,2,3,4);
IPAddress gateway(1,2,3,100);
IPAddress subnet(255,255,255,0);

unsigned long ulReqcount;

const int LED = LED_BUILTIN;

String newMessage;
char sMsg[32];


// Create an instance of the server on Port 80
WiFiServer server(80);

void demoHello() {
  oled.displayFill(0x00);  //  cls
  oled.displayFormat(2, oled.INVERSE);
  oled.displayPrintXY(64-14*3, 0, F(" Web1234Oled2 "));
  oled.displayFormat(2, oled.NORMAL);
  oled.displayPrintXY(0, 0, F("SSID: ESP-IP 1.2.3.4"));
  oled.displayPrintXY(0, 2, F("PASS: 12345678"));
  oled.displayPrintXY(0, 4, F("URL:  http://1.2.3.4"));
  oled.displayPrintXY(0, 6, F("     2018-10-19a    "));
}

void setup() {
  // setup globals
  ulReqcount=0; 
  
  // prepare LED GPIO
  pinMode(LED, OUTPUT);
  
  // start serial
  Serial.begin(115200);
  Serial.println("Start...");
  delay(1);
  
  // I2C Init
  Wire.begin();
  #ifdef __AVR__
    TWBR=IIC400KHZCLOCK; // 400kHz I2C Speed (AVR 8 Bit MCU)
  #else
    //Wire.setClock(400000); // 400kHz I2C Speed
    Wire.setClock(800000); // 800kHz I2C Speed
  #endif
  
  // OLED
  oled.D1306_begin(D1306_I2CADDRESS);  // initialize with the I2C addr
  delay(1000);
  
  // Scroll large Message
//  oled._charStretchX = 3;
//  oled._charStretchY = 4;
  oled.displayScale(3,4);
  oled.scrollText("Web1234Oled2",2);
  oled.displayFill(0x00);
  
  // AP mode
  digitalWrite(LED, LOW); // LED ON
  Serial.print("Set softAPConfig ... ");
  Serial.println(WiFi.softAPConfig(local_ip, gateway, subnet) ? "Ready" : "Failed");
  Serial.print("Set softAP ... ");
  Serial.println(WiFi.softAP(ssid, password) ? "Ready" : "Failed");
  Serial.print("softAP IP Address = ");
  Serial.println(WiFi.softAPIP());
  server.begin();
  digitalWrite(LED, HIGH); // LED OFF
  
  // hello screen
  digitalWrite(LED_BUILTIN, LOW);   // Turn the LED on
  delay(100);
  digitalWrite(LED_BUILTIN, HIGH);  // Turn the LED off
  delay(1000);
  demoHello();
  delay(5000);
  oled.displayFill(0x00);  //  cls
  Serial.println("setup() complete");
}


void loop() 
{ 
  oled.displayFormat(2, oled.NORMAL);
  //displayClrLine(0);
  oled.displayXYUInt(0,0,ulReqcount,2);
  oled.displayFormat(3, oled.NORMAL);
  oled.displayPrintXY(12, 0, F(">"));
  // Check if a client has connected
  WiFiClient client = server.available();
  if (!client) 
  {
    return;    
  }
  
  // Wait until the client sends some data
  Serial.println("new client");
  unsigned long ultimeout = millis()+250;
  while(!client.available() && (millis()<ultimeout) )
  {
    delay(1);
  }
  if(millis()>ultimeout) 
  {
    Serial.println("client connection time-out!");
    return; 
  }
  
  // Read the first line of the request
  String sRequest = client.readStringUntil('\r');
  //Dump
  Serial.println(sRequest);
  client.flush();
  
  // stop client, if request is empty
  if(sRequest=="")
  {
    Serial.println("empty request! - stopping client");
    client.stop();
    return;
  }
  
  // get path; end of path is either space or ?
  // Syntax is e.g. GET /?pin=MOTOR1STOP HTTP/1.1
  String sPath="",sParam="", sCmd="";
  String sGetstart="GET ";
  int iStart,iEndSpace,iEndQuest;
  iStart = sRequest.indexOf(sGetstart);
  if (iStart>=0)
  {
    iStart+=+sGetstart.length();
    iEndSpace = sRequest.indexOf(" ",iStart);
    iEndQuest = sRequest.indexOf("?",iStart);
    
    // are there parameters?
    if(iEndSpace>0)
    {
      if(iEndQuest>0)
      {
        // there are parameters
        sPath  = sRequest.substring(iStart,iEndQuest);
        sParam = sRequest.substring(iEndQuest,iEndSpace);
      }
      else
      {
        // NO parameters
        sPath  = sRequest.substring(iStart,iEndSpace);
      }
    }
  }
  
  ///////////////////////////////////////////////////////////////////////////////
  // output parameters to serial, you may connect e.g. an Arduino and react on it
  ///////////////////////////////////////////////////////////////////////////////
  if(sParam.length()>0)
  {
    int iEqu=sParam.indexOf("=");
    if(iEqu>=0)
    {
      sCmd = sParam.substring(iEqu+1,sParam.length());
      Serial.print("sPath  ");
      Serial.println(sPath);
      Serial.print("sParam ");
      Serial.println(sParam);
      Serial.print("sCmd   ");
      Serial.println(sCmd);
    }
  }
  
  
  ///////////////////////////
  // format the html response
  ///////////////////////////
  String sResponse,sHeader;
  
  ////////////////////////////
  // 404 for non-matching path
  ////////////////////////////
  if(sPath!="/")
  {
    sResponse="<html><head><title>404 Not Found</title></head><body><h1>Not Found</h1><p>The requested URL was not found on this server.</p></body></html>";
    
    sHeader  = "HTTP/1.1 404 Not found\r\n";
    sHeader += "Content-Length: ";
    sHeader += sResponse.length();
    sHeader += "\r\n";
    sHeader += "Content-Type: text/html\r\n";
    sHeader += "Connection: close\r\n";
    sHeader += "\r\n";
    Serial.print("404 Not found: ");
    Serial.println(sPath);
  }
  ///////////////////////
  // format the html page
  ///////////////////////
  else
  {
    ulReqcount++;
    sResponse  = "<html><head><title>ESP8266 DEMO</title></head><body>";
    sResponse += "<font color=\"#000000\"><body bgcolor=\"#eeeeee\">";
    sResponse += "<meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0, user-scalable=yes\">";
    sResponse += "<center>";
    sResponse += "<h1>Demo f&uumlr ESP8266 Access Point</h1>";
    sResponse += "Schalte die LED ein und aus oder sende eine Nachricht.<BR>";
    sResponse += "<FONT SIZE=+1>";
    sResponse += "<p>LED <a href=\"?pin=LED_ON\"><button>einschalten</button></a>&nbsp;<a href=\"?pin=LED_OFF\"><button>ausschalten</button></a></p>";
    
    //////////////////////
    // react on parameters
    //////////////////////
    if (sCmd.length()>0)
    {
      // write received command to html page
      sResponse += "Kommando: " + sCmd + "<br>";
      sCmd.toCharArray(sMsg, 11);
      oled.displayClrLine(2);
      oled.displayPrintXY(0, 2, sMsg);
      
      // switch GPIO
      if(sCmd.indexOf("LED_ON")>=0) {
        digitalWrite(LED, LOW); // LED ON
      }
      else if(sCmd.indexOf("LED_OFF")>=0) {
        digitalWrite(LED, HIGH); // LED OFF
      }
      
      // read message
      sMsg[0]=0;
      if(sParam.indexOf("Msg1")>=0) {
        if(sCmd.indexOf("MSGYO")>=0) {
          newMessage = "YO!";
        }
        else if(sCmd.indexOf("MSGNO")>=0) {
          newMessage = "NO!";
        }
        else {
          newMessage = sCmd; // store message
        }
        newMessage.toCharArray(sMsg, 32);
      }
    }
    
    sResponse += "Nachricht: "+newMessage+"<br>";
    sResponse += "<form method=""""get"""" autocomplete=""""off"""">";
    sResponse += "<input name=""""Msg1"""" size=""""20"""" maxlength=""""20"""">";
    sResponse += "<br><button type=""""submit"""">Absenden</button></form>";
    sResponse += "<a href=\"?Msg1=MSGYO\"><button>--- YO ---</button></a>&nbsp;"
                 "<a href=\"?Msg1=MSGNO\"><button>--- NO ---</button></a>";
    sResponse += "</center>";
    sResponse += "";
    
    sResponse += "<FONT SIZE=-2>";
    sResponse += "<BR>Aufrufz&auml;hler="; 
    sResponse += ulReqcount;
    sResponse += "<BR>";
    sResponse += "Webserver Vorlage von Stefan Thesen 08/2015<BR>";
    sResponse += "</body></html>";
    
    sHeader  = "HTTP/1.1 200 OK\r\n";
    sHeader += "Content-Length: ";
    sHeader += sResponse.length();
    sHeader += "\r\n";
    sHeader += "Content-Type: text/html\r\n";
    sHeader += "Connection: close\r\n";
    sHeader += "\r\n";
  }
  
  // Send the response to the client
  client.print(sHeader);
  client.print(sResponse);
  
  // and stop the client
  delay(10);
  client.stop();
  Serial.println("Client disonnected");
  
  // print newMessage to OLED
  oled.displayFormat(5, oled.NORMAL);
  oled.scrollText(sMsg,4);
  // delete message
  sMsg[0]=0;
}


